@@ -1,5 +1,5 @@
 /* Keyboard Accessibility Status Applet
- * Copyright 2003 Sun Microsystems Inc.
+ * Copyright 2003, 2004 Sun Microsystems Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -26,6 +26,7 @@
 #include <gdk/gdkx.h>
 #include <X11/XKBlib.h>
 #define XK_MISCELLANY
+#define XK_XKB_KEYS
 #include <X11/keysymdef.h>
 #include "applet.h"
 
@@ -108,6 +109,8 @@ static ButtonIconStruct button_icons [] = {
 	{Button3Mask, MOUSEKEYS_BUTTON_RIGHT}
 };
 
+static void popup_error_dialog (AccessxStatusApplet        *sapplet);
+
 /* cribbed from geyes */
 static void
 about_cb (BonoboUIComponent          *uic,
@@ -133,6 +136,9 @@ about_cb (BonoboUIComponent          *uic,
 	const gchar *translator_credits = _("translator_credits");
 
 	if (sapplet->about_dialog) {
+		gtk_window_set_screen (GTK_WINDOW (sapplet->about_dialog),
+				       gtk_widget_get_screen (GTK_WIDGET (sapplet->applet)));
+
 		gtk_window_present (GTK_WINDOW (sapplet->about_dialog));
 		return;
 	}
@@ -207,20 +213,26 @@ dialog_cb (BonoboUIComponent *component,
 	   AccessxStatusApplet *sapplet,
 	   const char        *verb)
 {
-	GError *err = NULL;
+	GError *error = NULL;
 	int ret;
 
-	ret = egg_screen_execute_shell (gtk_widget_get_screen (GTK_WIDGET (sapplet->applet)),
-					NULL, "gnome-accessibility-keyboard-properties", &err);
 
-	if (ret < 0) {
+	if (sapplet->error_type != ACCESSX_STATUS_ERROR_NONE) {
+		popup_error_dialog (sapplet);
+		return;
+	}
+
+	gdk_spawn_command_line_on_screen (gtk_widget_get_screen (GTK_WIDGET (sapplet->applet)),
+			"gnome-accessibility-keyboard-properties", &error);
+
+	if (error != NULL) {
 		GtkWidget *dialog = 
 			gtk_message_dialog_new (NULL,
 						GTK_DIALOG_DESTROY_WITH_PARENT,
 						GTK_MESSAGE_ERROR,
 						GTK_BUTTONS_CLOSE,
 						_("There was an error launching the keyboard capplet : %s"), 
-						err->message);
+						error->message);
 
 		g_signal_connect (G_OBJECT (dialog),
 				  "response",
@@ -231,16 +243,16 @@ dialog_cb (BonoboUIComponent *component,
 		gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
 
 		gtk_widget_show (dialog);
-		g_error_free (err);
+		g_error_free (error);
 	}
 }
 
 static const BonoboUIVerb accessx_status_applet_menu_verbs [] = {
-        BONOBO_UI_UNSAFE_VERB ("Dialog", dialog_cb),
-        BONOBO_UI_UNSAFE_VERB ("Help", help_cb),
-        BONOBO_UI_UNSAFE_VERB ("About", about_cb),
+	BONOBO_UI_UNSAFE_VERB ("Dialog", dialog_cb),
+	BONOBO_UI_UNSAFE_VERB ("Help", help_cb),
+	BONOBO_UI_UNSAFE_VERB ("About", about_cb),
 
-        BONOBO_UI_VERB_END
+	BONOBO_UI_VERB_END
 };
 
 XkbDescPtr 
@@ -257,14 +269,14 @@ accessx_status_applet_get_xkb_desc (AccessxStatusApplet *sapplet)
 					  &reason_return);
 		g_assert (display); /* TODO: change error message below to something user-viewable */
 		if (display == NULL)
-		        g_warning ("Xkb extension could not be initialized! (error code %x)", reason_return);
+			g_warning ("Xkb extension could not be initialized! (error code %x)", reason_return);
 		else 
 			sapplet->xkb = XkbGetMap (display, 
 						  XkbAllComponentsMask,
 						  XkbUseCoreKbd);
 		g_assert (sapplet->xkb);
 		if (sapplet->xkb == NULL)
-		        g_warning ("Xkb keyboard description not available!");
+			g_warning ("Xkb keyboard description not available!");
 		sapplet->xkb_display = display;
 	}
 	return sapplet->xkb;
@@ -286,7 +298,10 @@ accessx_status_applet_xkb_select (AccessxStatusApplet *sapplet)
 					  XkbAllEventsMask, 
 					  XkbAllEventsMask);
 		sapplet->xkb = accessx_status_applet_get_xkb_desc (sapplet);
+	} else {
+		sapplet->error_type = ACCESSX_STATUS_ERROR_XKB_DISABLED;
 	}
+
 	return retval;
 }
 
@@ -309,8 +324,11 @@ accessx_status_applet_init_modifiers (AccessxStatusApplet *sapplet)
 	super_mask = XkbKeysymToModifiers (sapplet->xkb_display, XK_Super_L);
 	if (super_mask) gtk_widget_show (sapplet->super_indicator);
 	else gtk_widget_hide (sapplet->super_indicator);
+	alt_gr_mask = XkbKeysymToModifiers (sapplet->xkb_display, XK_Mode_switch) | 
+	    XkbKeysymToModifiers (sapplet->xkb_display, XK_ISO_Level3_Shift) |
+	    XkbKeysymToModifiers (sapplet->xkb_display, XK_ISO_Level3_Latch) | 
+	    XkbKeysymToModifiers (sapplet->xkb_display, XK_ISO_Level3_Lock);
 
-	alt_gr_mask = XkbKeysymToModifiers (sapplet->xkb_display, XK_Mode_switch);
 	if (alt_gr_mask) gtk_widget_show (sapplet->alt_graph_indicator);
 	else gtk_widget_hide (sapplet->alt_graph_indicator);
 
@@ -327,7 +345,6 @@ accessx_status_applet_init_modifiers (AccessxStatusApplet *sapplet)
 			modifiers[i].indicator = sapplet->hyper_indicator;
 		else if (modifiers[i].mask == super_mask)
 			modifiers[i].indicator = sapplet->super_indicator;
-		/* due to some XKB/XFree weirdness, AltGr doesn't give notifications */
 		else if (modifiers[i].mask == alt_gr_mask)
 			modifiers[i].indicator = sapplet->alt_graph_indicator;
 	}
@@ -393,10 +410,9 @@ accessx_status_applet_get_glyph_pixbuf (AccessxStatusApplet *sapplet,
 	cmap = gdk_drawable_get_colormap (GDK_DRAWABLE (pixmap));
 	if (!cmap) cmap = gdk_colormap_new (visual, FALSE);
 	else g_object_ref (cmap);
-	gdk_color_white (cmap, &white);
 	gdk_colormap_alloc_color (cmap, fg, FALSE, TRUE);
 	gdk_colormap_alloc_color (cmap, bg, FALSE, TRUE);
-	gdk_gc_set_foreground (gc, &white);
+	gdk_gc_set_foreground (gc, bg);
 	gdk_draw_rectangle (pixmap, gc, True, 0, 0, w, h);
 	gdk_gc_set_foreground (gc, fg);
 	gdk_gc_set_background (gc, bg);
@@ -411,7 +427,7 @@ accessx_status_applet_get_glyph_pixbuf (AccessxStatusApplet *sapplet,
 	glyph_pixbuf = gdk_pixbuf_get_from_drawable (NULL, pixmap, 
 						     cmap, 0, 0, 0, 0, w, h);
 	gdk_pixmap_unref (pixmap);
-	alpha_pixbuf = gdk_pixbuf_add_alpha (glyph_pixbuf, TRUE, 255, 255, 255);
+	alpha_pixbuf = gdk_pixbuf_add_alpha (glyph_pixbuf, TRUE, bg->red >> 8, bg->green >> 8, bg->blue >> 8);
 	g_object_unref (G_OBJECT (glyph_pixbuf));
 	g_object_unref (G_OBJECT (cmap));
 	g_object_unref (G_OBJECT (gc));
@@ -811,6 +827,7 @@ accessx_status_applet_altgraph_icon_set (AccessxStatusApplet *sapplet, GtkWidget
 	GtkStyle    *style = gtk_widget_get_style (widget);
 	GdkPixbuf   *icon_base;
 
+	gtk_widget_set_sensitive (widget, TRUE);
 	for (i = 0; i < 3; ++i) {
 		int alpha;
 		GdkColor *fg, *bg;
@@ -822,38 +839,42 @@ accessx_status_applet_altgraph_icon_set (AccessxStatusApplet *sapplet, GtkWidget
 		switch (states[i]) {
 		case GTK_STATE_NORMAL:
 			alpha = 255;
+			gtk_widget_set_sensitive (widget, TRUE);
 			gtk_widget_set_state (widget, GTK_STATE_NORMAL);
 			break;
 		case GTK_STATE_SELECTED:
-			/* 
-			 * This is incorrect for theming, but since XKB doesn't generate 
-			 * latch events properly, 
-			 * for AltGr, it doesn't matter...  this state isn't ever used.
-			 */
-			fg = &style->base[states[i]]; /* err, what's that again? */
+		    /* FIXME: should use text/base here, for selected ? */
+			fg = &style->white; 
 			bg = &style->black;
 			alpha = 255;
+			gtk_widget_set_sensitive (widget, TRUE);
 			gtk_widget_set_state (widget, GTK_STATE_SELECTED);
 			break;
 		case GTK_STATE_INSENSITIVE:
 		default:
 			alpha = 63;
 			gtk_widget_set_sensitive (widget, FALSE);
+			wildcarded = TRUE;
 			break;
 		}
 		icon_base = gtk_widget_render_icon (widget,
 			ACCESSX_BASE_ICON, icon_size_spec, NULL);
 		pixbuf = gdk_pixbuf_copy (icon_base);
 		g_object_unref (icon_base);
+	       /* 
+		* should be N_("ae"));
+		* need en_ locale for this.
+		*/
+	       /* 
+		* Translators: substitute an easily-recognized single glyph 
+		* from Level 2, i.e. an AltGraph character from a common keyboard 
+		* in your locale. 
+		*/
 		glyph_pixbuf = accessx_status_applet_get_glyph_pixbuf (sapplet, 
 								       widget,
 								       pixbuf, 
 								       fg, 
 								       bg,
-				       /* 
-					* should be N_("ae"));
-					* need en_ locale for this
-					*/
 								       ("æ"));
 		gdk_pixbuf_composite (glyph_pixbuf, pixbuf, 0, 0, 
 				      gdk_pixbuf_get_width (glyph_pixbuf),
@@ -973,22 +994,63 @@ accessx_status_applet_layout_box (AccessxStatusApplet *sapplet, GtkWidget *box,
 
 	gtk_widget_show (sapplet->box);
 	gtk_widget_show (GTK_WIDGET (sapplet->applet));
-	if (GTK_WIDGET_REALIZED (sapplet->applet))
+	if (GTK_WIDGET_REALIZED (sapplet->box) &&
+            sapplet->initialized)
 		accessx_status_applet_update (sapplet, ACCESSX_STATUS_ALL, NULL);
 }
 
-static void
-accessx_status_applet_realize (GtkWidget *widget, gpointer user_data)
+static void 
+disable_applet (AccessxStatusApplet* sapplet)
 {
-	AccessxStatusApplet *sapplet = user_data;
-	if (!sapplet->initialized) {
-		sapplet->initialized = True;
-		accessx_status_applet_xkb_select (sapplet);
-		gdk_window_add_filter (NULL, accessx_status_xkb_filter, sapplet);
+	gtk_widget_hide (sapplet->meta_indicator);
+	gtk_widget_hide (sapplet->hyper_indicator);
+	gtk_widget_hide (sapplet->super_indicator);
+	gtk_widget_hide (sapplet->alt_graph_indicator);
+	gtk_widget_hide (sapplet->shift_indicator);
+	gtk_widget_hide (sapplet->ctrl_indicator);
+	gtk_widget_hide (sapplet->alt_indicator);
+	gtk_widget_hide (sapplet->meta_indicator);
+	gtk_widget_hide (sapplet->mousefoo);
+	gtk_widget_hide (sapplet->stickyfoo);
+	gtk_widget_hide (sapplet->slowfoo);
+	gtk_widget_hide (sapplet->bouncefoo);
+}
+
+static void 
+popup_error_dialog (AccessxStatusApplet* sapplet)
+{
+	GtkWidget *dialog;
+	gchar *error_txt;
+
+	switch (sapplet->error_type) {
+		case ACCESSX_STATUS_ERROR_XKB_DISABLED : 
+			error_txt = g_strdup (_("XKB Extenstion is not enabled"));
+			break;
+
+		case ACCESSX_STATUS_ERROR_UNKNOWN :
+
+		default	: error_txt = g_strdup (_("Unknown error"));
+			  break;				
 	}
-	accessx_status_applet_init_modifiers (sapplet);
-	accessx_status_applet_update (sapplet, ACCESSX_STATUS_ALL, NULL);
-	return;
+
+	dialog = gtk_message_dialog_new (NULL,
+				   	 GTK_DIALOG_DESTROY_WITH_PARENT,
+					 GTK_MESSAGE_ERROR,
+					 GTK_BUTTONS_CLOSE,
+					 _("Error: %s"),
+					 error_txt);
+
+	g_signal_connect (G_OBJECT (dialog),
+			  "response",
+			  G_CALLBACK (gtk_widget_destroy), NULL);
+
+	gtk_window_set_screen (GTK_WINDOW (dialog),
+	gtk_widget_get_screen (GTK_WIDGET (sapplet->applet)));
+
+	gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
+
+	gtk_widget_show (dialog);
+	g_free (error_txt);
 }
 
 static AccessxStatusApplet *
@@ -999,11 +1061,12 @@ create_applet (PanelApplet *applet)
 	AtkObject           *atko;
 	GdkPixbuf	    *pixbuf;
         gint                 large_toolbar_pixels;
-		
+
 	sapplet->xkb = NULL;
 	sapplet->xkb_display = NULL;
 	sapplet->box = NULL;
 	sapplet->initialized = False; /* there must be a better way */
+	sapplet->error_type = ACCESSX_STATUS_ERROR_NONE;
 	sapplet->applet = applet;
 	panel_applet_set_flags (applet, PANEL_APPLET_EXPAND_MINOR);
 	sapplet->orient = panel_applet_get_orient (applet);
@@ -1115,10 +1178,38 @@ accessx_status_applet_resize (GtkWidget *widget, int size, gpointer user_data)
 	; /* TODO: either rescale icons to fit panel, or tile them when possible */
 }
 
+static void
+accessx_status_applet_background (PanelApplet *a, PanelAppletBackgroundType type,
+				  GdkColor *color, GdkPixmap *pixmap, gpointer user_data)
+{
+	AccessxStatusApplet *sapplet = user_data;
+	GtkRcStyle *rc_style = gtk_rc_style_new ();
+
+	switch (type) {
+	case PANEL_PIXMAP_BACKGROUND:
+		gtk_widget_modify_style (GTK_WIDGET (sapplet->applet), rc_style);
+		break;
+
+	case PANEL_COLOR_BACKGROUND:
+		gtk_widget_modify_bg (GTK_WIDGET (sapplet->applet), GTK_STATE_NORMAL, color);
+		break;
+
+	case PANEL_NO_BACKGROUND:
+		gtk_widget_modify_style (GTK_WIDGET (sapplet->applet), rc_style);
+		break;
+
+	default:
+		gtk_widget_modify_style (GTK_WIDGET (sapplet->applet), rc_style);
+		break;
+	}
+
+	gtk_rc_style_unref (rc_style);
+}
+
 static gboolean
 button_press_cb (GtkWidget *widget, GdkEventButton *event, AccessxStatusApplet *sapplet)
 {
-	if (event->button == 1 && event->type == GDK_2BUTTON_PRESS) 
+	if (event->button == 1 && event->type == GDK_BUTTON_PRESS) 
 		dialog_cb (NULL, sapplet, NULL);
 
 	return FALSE;
@@ -1154,22 +1245,59 @@ accessx_status_applet_reset (gpointer user_data)
 	return FALSE;
 }
 
+static gboolean 
+accessx_status_applet_initialize (AccessxStatusApplet *sapplet)
+{
+	if (!sapplet->initialized) {
+		sapplet->initialized = True;
+		if (!accessx_status_applet_xkb_select (sapplet)) {
+			disable_applet (sapplet);
+			popup_error_dialog (sapplet);
+			return FALSE ;
+		}
+		gdk_window_add_filter (NULL, accessx_status_xkb_filter, sapplet);
+	}
+	accessx_status_applet_init_modifiers (sapplet);
+	accessx_status_applet_update (sapplet, ACCESSX_STATUS_ALL, NULL);
+
+	return TRUE;
+}
+
+static void
+accessx_status_applet_realize (GtkWidget *widget, gpointer user_data)
+{
+	AccessxStatusApplet *sapplet = user_data;
+
+	if (!accessx_status_applet_initialize (sapplet))
+		return;
+
+	g_idle_add (accessx_status_applet_reset, sapplet);
+	return;
+}
+
 static gboolean
 accessx_status_applet_fill (PanelApplet *applet)
 {
 	AccessxStatusApplet *sapplet;
 	AtkObject           *atk_object;
+	gboolean was_realized = FALSE;
 
 	sapplet = create_applet (applet);
 
-	g_signal_connect_after (G_OBJECT (sapplet->box), 
-				"realize", G_CALLBACK (accessx_status_applet_realize), 
-				sapplet);
+	if (!GTK_WIDGET_REALIZED (sapplet->box)) {
+		g_signal_connect_after (G_OBJECT (sapplet->box), 
+					"realize", G_CALLBACK (accessx_status_applet_realize), 
+					sapplet);
+	} else {
+		accessx_status_applet_initialize (sapplet);
+		was_realized = TRUE;
+	}
 
 	g_object_connect (sapplet->applet,
 			  "signal::destroy", accessx_status_applet_destroy, sapplet,
 			  "signal::change_orient", accessx_status_applet_reorient, sapplet,
 			  "signal::change_size", accessx_status_applet_resize, sapplet,
+			  "signal::change_background", accessx_status_applet_background, sapplet,
 			  NULL);
 			  
 	g_signal_connect (sapplet->applet, "button_press_event",
@@ -1208,8 +1336,9 @@ accessx_status_applet_fill (PanelApplet *applet)
 	atk_object_set_description (atk_object,
 				    _("Displays current state of keyboard accessibility features"));
 	gtk_widget_show_all (GTK_WIDGET (sapplet->applet));
-	/* hack for race insurance */
-	g_timeout_add (1000, accessx_status_applet_reset, sapplet);
+	if (was_realized) {
+		accessx_status_applet_reset (sapplet);
+	}
 
 	return TRUE;
 }
