@@ -52,6 +52,8 @@
 #define CDPLAYER_NEXT           "media-next"
 #define CDPLAYER_EJECT          "media-eject"
 
+#define GCONF_DEVICE_KEY        "/apps/gnome-cd/device"
+
 /* Function prototypes */
 static gboolean cdplayer_applet_factory (PanelApplet *applet, const gchar *iid, gpointer data);
 static gboolean applet_fill (PanelApplet *applet);
@@ -145,14 +147,24 @@ applet_fill (PanelApplet *applet)
     BonoboUIComponent *component;
     GtkTooltips *tooltips;
     int err;
+    GtkIconInfo *icon_info;
 
-    gnome_window_icon_set_default_from_file (GNOME_ICONDIR"/gnome-cdplayer-icon.png");
+    icon_info = gtk_icon_theme_lookup_icon (gtk_icon_theme_get_default (), "gnome-cdplayer-icon", 48, 0);
+    if (icon_info) {
+        gnome_window_icon_set_default_from_file (gtk_icon_info_get_filename (icon_info));
+        gtk_icon_info_free (icon_info);
+    }
     panel_applet_set_flags (applet, PANEL_APPLET_EXPAND_MINOR);
     cdplayer_init_stock_icons ();
       
     cd = g_new0(CDPlayerData, 1);
+    cd->gconf_client = gconf_client_get_default ();
     cd->panel.applet = GTK_WIDGET (applet);
-    
+
+    cd->about_dialog = NULL;
+    cd->error_busy_dialog = NULL;
+    cd->error_io_dialog = NULL;
+
     /* the rest of the widgets go in here */
     cdplayer = cd->panel.frame = gtk_hbox_new(FALSE, 0);
    
@@ -256,29 +268,11 @@ show_error (CDPlayerData *cd)
     gtk_widget_show (dialog);
 }
 
-static gboolean
-key_writable (PanelApplet *applet, const char *key)
-{
-	gboolean writable;
-	char *fullkey;
-	static GConfClient *client = NULL;
-	if (client == NULL)
-		client = gconf_client_get_default ();
-
-	fullkey = panel_applet_gconf_get_full_key (applet, key);
-
-	writable = gconf_client_key_is_writable (client, fullkey, NULL);
-
-	g_free (fullkey);
-
-	return writable;
-}
-
 static void
 cdplayer_load_config(CDPlayerData *cd)
 {
     g_free(cd->devpath);
-    cd->devpath = panel_applet_gconf_get_string(PANEL_APPLET(cd->panel.applet), "device_path", NULL);
+    cd->devpath = gconf_client_get_string(cd->gconf_client, GCONF_DEVICE_KEY, NULL);
     if (!cd->devpath || !strcmp(cd->devpath, "none"))
     {
         g_free(cd->devpath);
@@ -289,8 +283,8 @@ cdplayer_load_config(CDPlayerData *cd)
 static void
 cdplayer_save_config(CDPlayerData *cd)
 {
-    if (key_writable (PANEL_APPLET (cd->panel.applet), "device_path"))
-        panel_applet_gconf_set_string(PANEL_APPLET(cd->panel.applet), "device_path", cd->devpath, NULL);
+    if (gconf_client_key_is_writable (cd->gconf_client, GCONF_DEVICE_KEY, NULL))
+        gconf_client_set_string (cd->gconf_client, GCONF_DEVICE_KEY, cd->devpath, NULL);
 }
 
 static void
@@ -325,10 +319,21 @@ cdplayer_destroy(GtkWidget * widget, gpointer data)
         g_free(cd->track_description);
 
     g_free (cd->devpath);
+
+    if (cd->about_dialog)
+      gtk_widget_destroy (cd->about_dialog);
  
     if (cd->prefs_dialog)
       gtk_widget_destroy (cd->prefs_dialog);
 
+    if (cd->error_busy_dialog)
+      gtk_widget_destroy (cd->error_busy_dialog);
+
+    if (cd->error_io_dialog)
+      gtk_widget_destroy (cd->error_io_dialog);
+
+    g_object_unref (cd->gconf_client);
+
     cd->devpath = NULL;
     g_free(cd);
 }
@@ -361,8 +366,10 @@ start_gtcd_cb (BonoboUIComponent *component,
 {
     GError *error = NULL;
 
-    egg_screen_execute_command_line_async (
-		gtk_widget_get_screen (cd->panel.applet), "gnome-cd", &error);
+    gdk_spawn_command_line_on_screen (
+		    		gtk_widget_get_screen (GTK_WIDGET (cd->panel.applet)),
+				"gnome-cd", &error);
+
     if (error) {
 	GtkWidget *dialog;
 
@@ -525,7 +532,7 @@ preferences_cb (BonoboUIComponent *component,
     g_object_set_data (G_OBJECT (button), "entry", entry);
     gtk_widget_show (button);
 
-    if ( ! key_writable (PANEL_APPLET (cd->panel.applet), "device_path")) {
+    if ( ! gconf_client_key_is_writable (cd->gconf_client, GCONF_DEVICE_KEY, NULL)) {
 	    gtk_widget_set_sensitive (label, FALSE);
 	    gtk_widget_set_sensitive (entry, FALSE);
 	    gtk_widget_set_sensitive (button, FALSE);
@@ -570,10 +577,8 @@ about_cb (BonoboUIComponent *component,
 	  CDPlayerData      *cd,
 	  const char        *verb)
 {
-    static GtkWidget   *about     = NULL;
     GdkPixbuf	       *pixbuf;
     GError	       *error     = NULL;
-    gchar	       *file;
 
     static const gchar *authors[] =
     {
@@ -585,44 +590,47 @@ about_cb (BonoboUIComponent *component,
 
     const gchar *documenters[] =
     {
-	    NULL
+        "Chris Lyttle <chris@wilddev.net>",
+	"Sun GNOME Documentation Team <gdocteam@sun.com>",
+	NULL
     };
 
     const gchar *translator_credits = _("translator_credits");
 
-    if (about) {
-	gtk_window_set_screen (GTK_WINDOW (about),
+    if (cd->about_dialog) {
+	gtk_window_set_screen (GTK_WINDOW (cd->about_dialog),
 			       gtk_widget_get_screen (cd->panel.applet));
-	gtk_window_present (GTK_WINDOW (about));
+
+	gtk_window_present (GTK_WINDOW (cd->about_dialog));
         return;
     }
     
-    file = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_PIXMAP, "gnome-cdplayer-icon.png", FALSE, NULL);
-    pixbuf = gdk_pixbuf_new_from_file (file, &error);
-    g_free (file);
+    pixbuf = gtk_icon_theme_load_icon (gtk_icon_theme_get_default (), "gnome-cdplayer-icon", 48, 0, &error);
     
     if (error) {
-    	g_warning (G_STRLOC ": cannot open %s: %s", file, error->message);
+    	g_warning (G_STRLOC ": cannot open icon %s: %s", "gnome-cdplayer-icon", error->message);
 	g_error_free (error);
     }
     
-    about = gnome_about_new (_("CD Player"), VERSION,
-                             _("(C) 1997 The Free Software Foundation\n" \
-                               "(C) 2001 Chris Phelps (GNOME 2 Port)"),
-                             _("The CD Player applet is a simple audio CD player for your panel"),
-                             authors,
-			     documenters,
-			     strcmp (translator_credits, "translator_credits") != 0 ? translator_credits : NULL,
-			     pixbuf);
+    cd->about_dialog = gnome_about_new (_("CD Player"), VERSION,
+                                        _("(C) 1997 The Free Software Foundation\n" \
+                                          "(C) 2001 Chris Phelps (GNOME 2 Port)"),
+                                        _("The CD Player applet is a simple audio CD player for your panel"),
+                                        authors,
+                                        documenters,
+                                        strcmp (translator_credits, "translator_credits") != 0 ? translator_credits : NULL,
+                                        pixbuf);
     if (pixbuf)
     	gdk_pixbuf_unref (pixbuf);
 
-    gtk_window_set_wmclass (GTK_WINDOW (about), "cd player", "CD Player");
-    gtk_window_set_screen (GTK_WINDOW (about),
-			   gtk_widget_get_screen (cd->panel.applet));
-    g_signal_connect (G_OBJECT(about), "destroy",
-                      G_CALLBACK(gtk_widget_destroyed), &about);
-    gtk_widget_show (about);
+    gtk_window_set_wmclass (GTK_WINDOW (cd->about_dialog), "cd player", "CD Player");
+    gtk_window_set_screen (GTK_WINDOW (cd->about_dialog),
+                           gtk_widget_get_screen (cd->panel.applet));
+
+    g_signal_connect (G_OBJECT(cd->about_dialog), "destroy",
+                      G_CALLBACK(gtk_widget_destroyed), &cd->about_dialog);
+
+    gtk_widget_show (cd->about_dialog);
 }
 
 static void
@@ -943,7 +951,7 @@ cdplayer_play_pause(GtkWidget * w, gpointer data)
                    cd->played_by_applet = 1;
             break;
         }
-    } else if(status == DISC_TRAY_OPEN) {
+    } else if (status == DISC_TRAY_OPEN) {
         cdrom_load(cd->cdrom_device);
         cdrom_read_track_info(cd->cdrom_device);
         ret = cdrom_play(cd->cdrom_device, cd->cdrom_device->track0,
@@ -953,20 +961,57 @@ cdplayer_play_pause(GtkWidget * w, gpointer data)
     }
 
     if (ret == DISC_DEVICE_BUSY) {
-		GtkWidget *dialog;
-		dialog = gtk_message_dialog_new (NULL,
-						 GTK_DIALOG_DESTROY_WITH_PARENT,
-						 GTK_MESSAGE_ERROR,
-						 GTK_BUTTONS_OK,
-						 _("Audio device is busy, or being used by another application"),
-						 NULL);
-		gtk_window_set_screen (GTK_WINDOW (dialog),
-				       gtk_widget_get_screen (cd->panel.applet));
-		g_signal_connect_swapped (GTK_OBJECT (dialog), "response",
-		                          G_CALLBACK (gtk_widget_destroy),
-					  GTK_OBJECT (dialog));
+	    if (cd->error_busy_dialog) {
+		    gtk_window_set_screen (GTK_WINDOW (cd->error_busy_dialog),
+				    gtk_window_get_screen (GTK_WINDOW (cd->panel.applet)));
 
-		gtk_widget_show_all (dialog);
+		    gtk_window_present (GTK_WINDOW (cd->error_busy_dialog));
+
+		    return;
+	    }
+
+	    cd->error_busy_dialog = gtk_message_dialog_new (NULL,
+			    GTK_DIALOG_DESTROY_WITH_PARENT,
+			    GTK_MESSAGE_ERROR,
+			    GTK_BUTTONS_OK,
+			    _("Audio device is busy, or being used by another application"),
+			    NULL);
+
+	    gtk_window_set_screen (GTK_WINDOW (cd->error_busy_dialog),
+			    gtk_widget_get_screen (cd->panel.applet));
+
+	    g_signal_connect_swapped (GTK_OBJECT (cd->error_busy_dialog), "response",
+			    G_CALLBACK (gtk_widget_destroy),
+			    GTK_OBJECT (cd->error_busy_dialog));
+
+	    gtk_widget_show_all (cd->error_busy_dialog);
+    }
+
+    if (ret == DISC_IO_ERROR) {
+	    if (cd->error_io_dialog) {
+		    gtk_window_set_screen (GTK_WINDOW (cd->error_io_dialog),
+				    gtk_window_get_screen (GTK_WINDOW (cd->panel.applet)));
+
+		    gtk_window_present (GTK_WINDOW (cd->error_io_dialog));
+
+		    return;
+	    }
+
+	    cd->error_io_dialog = gtk_message_dialog_new (NULL,
+			    GTK_DIALOG_DESTROY_WITH_PARENT,
+			    GTK_MESSAGE_ERROR,
+			    GTK_BUTTONS_OK,
+			    _("No Device Found or Illegal Format"),
+			    NULL);
+
+	    gtk_window_set_screen (GTK_WINDOW (cd->error_io_dialog),
+			    gtk_widget_get_screen (cd->panel.applet));
+
+	    g_signal_connect_swapped (GTK_OBJECT (cd->error_io_dialog), "response",
+			    G_CALLBACK (gtk_widget_destroy),
+			    GTK_OBJECT (cd->error_io_dialog));
+
+	    gtk_widget_show_all (cd->error_io_dialog);
     }
 }
 
@@ -1078,13 +1123,23 @@ set_atk_name_description(GtkWidget *widget, const gchar *name,
     const gchar *description)
 {	
     AtkObject *aobj;
+    const gchar *old_name;
+    const gchar *old_description;
 	
     aobj = gtk_widget_get_accessible(widget);
     /* Check if gail is loaded */
     if (GTK_IS_ACCESSIBLE (aobj) == FALSE)
         return; 
-    atk_object_set_name(aobj, name);
-    atk_object_set_description(aobj, description);
+    old_name = atk_object_get_name (aobj);
+    old_description = atk_object_get_description (aobj);
+    if ((old_name && name && strcmp (old_name, name)) ||
+        (!old_name && name) ||
+        (old_name && !name))
+      atk_object_set_name(aobj, name);
+    if ((old_description && description && strcmp (old_description, description)) ||
+        (!old_description && description) ||
+        (old_description && !description))
+      atk_object_set_description(aobj, description);
 }
 
 static void
