@@ -311,32 +311,6 @@ menuitem_activated (GtkMenuItem *menuitem, charpick_data *curr_data)
 		panel_applet_gconf_set_string (applet, "current_list", curr_data->charlist, NULL);
 }
 
-static void
-add_palette (GtkMenuItem *menuitem, charpick_data *curr_data)
-{
-	PanelApplet *applet = PANEL_APPLET (curr_data->applet);
-	GList *list = curr_data->chartable;
-	gchar *new;
-	
-	new = run_edit_dialog (NULL, _("Add Palette"));
-	if (!new)
-		return;
-		
-	list = g_list_append (list, new);
-	if (curr_data->chartable == NULL) {
-		curr_data->chartable = list;
-		curr_data->charlist = curr_data->chartable->data;
-		build_table (curr_data);
-		if (key_writable (PANEL_APPLET (curr_data->applet), "current_list"))
-			panel_applet_gconf_set_string (PANEL_APPLET (curr_data->applet),
-						      "current_list", 
-					  	       curr_data->charlist, NULL);
-	}
-	
-	save_chartable (curr_data);
-  	populate_menu (curr_data);
-}
-
 void
 populate_menu (charpick_data *curr_data)
 {
@@ -364,19 +338,6 @@ populate_menu (charpick_data *curr_data)
 			gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (menuitem), TRUE);
 		list = g_list_next (list);
 	}
-	
-	menuitem = gtk_separator_menu_item_new ();
-	gtk_widget_show (menuitem);
-	gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
-	
-	menuitem = gtk_menu_item_new_with_label (_("Add a palette..."));
-	gtk_widget_show (menuitem);
-	g_signal_connect (G_OBJECT (menuitem), "activate",
-				   G_CALLBACK (add_palette), curr_data);
-	gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
-	if ( ! key_writable (PANEL_APPLET (curr_data->applet), "chartable"))
-		gtk_widget_set_sensitive (menuitem, FALSE);
-	
 }
 
 static void
@@ -417,10 +378,13 @@ chooser_button_clicked (GtkButton *button, charpick_data *curr_data)
 {
 	if (GTK_WIDGET_VISIBLE (curr_data->menu))
 		gtk_menu_popdown (GTK_MENU (curr_data->menu));
-	else
+	else {
+		gtk_menu_set_screen (GTK_MENU (curr_data->menu),
+				gtk_widget_get_screen (GTK_WIDGET (curr_data->applet)));
+		
 		gtk_menu_popup (GTK_MENU (curr_data->menu), NULL, NULL, get_menu_pos, curr_data,
-					    0, gtk_get_current_event_time());
-					    
+				0, gtk_get_current_event_time());
+	}				    
 }
 
 /* creates table of buttons, sets up their callbacks, and packs the table in
@@ -481,11 +445,14 @@ build_table(charpick_data *p_curr_data)
     gchar label[7];
     gint num;
     GtkRequisition req;
+    gchar atk_desc[50];
     
     g_utf8_strncpy (label, charlist, 1);
     charlist = g_utf8_next_char (charlist);
    
     toggle_button[i] = gtk_toggle_button_new_with_label (label);
+    sprintf(atk_desc, _("insert special character %s"), label);
+    set_atk_name_description (toggle_button[i], NULL, atk_desc);
     gtk_widget_show (toggle_button[i]);
     gtk_button_set_relief(GTK_BUTTON(toggle_button[i]), GTK_RELIEF_NONE);
     gtk_tooltips_set_tip (tooltips, toggle_button[i], 
@@ -505,18 +472,14 @@ build_table(charpick_data *p_curr_data)
                         (GtkSignalFunc) button_press_hack, p_curr_data->applet);
   }
   
-  switch (panel_applet_get_orient (PANEL_APPLET (p_curr_data->applet))) {
-     	case PANEL_APPLET_ORIENT_DOWN:
-     	case PANEL_APPLET_ORIENT_UP:
-        	size_ratio = p_curr_data->panel_size / max_height;
-        	button_box = gtk_vbox_new (TRUE, 0);
-        	break;
-     	case PANEL_APPLET_ORIENT_LEFT:
-     	case PANEL_APPLET_ORIENT_RIGHT:
-     		size_ratio = p_curr_data->panel_size / max_width;
-     		button_box = gtk_hbox_new (TRUE, 0);
-		break;
+  if (p_curr_data->panel_vertical) {
+    size_ratio = p_curr_data->panel_size / max_width;
+    button_box = gtk_hbox_new (TRUE, 0);
+  } else {
+    size_ratio = p_curr_data->panel_size / max_height;
+    button_box = gtk_vbox_new (TRUE, 0);
   }
+
   gtk_box_pack_start (GTK_BOX (box), button_box, TRUE, TRUE, 0);
   
   size_ratio = MAX (size_ratio, 1);
@@ -528,10 +491,10 @@ build_table(charpick_data *p_curr_data)
   }
   
   for (i = 0; i <len; i++) {  	
-  	float delta = len/(float)size_ratio;
+  	int delta = len/size_ratio;
   	int index;
   	
-  	index = (int) floor (i / delta);  	
+  	index = i / delta;
 	index = CLAMP (index, 0, size_ratio-1);	
   	gtk_box_pack_start (GTK_BOX (row_box[index]), toggle_button[i], TRUE, TRUE, 0);
   }
@@ -544,10 +507,18 @@ build_table(charpick_data *p_curr_data)
   
 }
 
-static void applet_change_pixel_size(PanelApplet *applet, gint size, gpointer data)
+static void applet_size_allocate(PanelApplet *applet, GtkAllocation *allocation, gpointer data)
 {
   charpick_data *curr_data = data;
-  curr_data->panel_size = size;
+  if (curr_data->panel_vertical) {
+    if (curr_data->panel_size == allocation->width)
+      return;
+    curr_data->panel_size = allocation->width;
+  } else {
+    if (curr_data->panel_size == allocation->height)
+      return;
+    curr_data->panel_size = allocation->height;
+  }
 
   build_table (curr_data);
   return;
@@ -571,7 +542,6 @@ about (BonoboUIComponent *uic,
        charpick_data     *curr_data,
        const char        *verb)
 {
-  static GtkWidget *about_box = NULL;
   GdkPixbuf   	   *pixbuf;
   GError      	   *error     = NULL;
   gchar            *file;
@@ -585,49 +555,56 @@ about (BonoboUIComponent *uic,
   };
 
   const gchar *documenters[] = {
+          "Dan Mueth <d-mueth@uchicago.edu>",
+          "Sun GNOME Documentation Team <gdocteam@sun.com>",
 	  NULL
   };
 
   const gchar *translator_credits = _("translator_credits");
 
-  if (about_box) {
-	gtk_window_set_screen (GTK_WINDOW (about_box),
+  if (curr_data->about_dialog) {
+	gtk_window_set_screen (GTK_WINDOW (curr_data->about_dialog),
 			       gtk_widget_get_screen (curr_data->applet));
-	gtk_window_present (GTK_WINDOW (about_box));
+
+	gtk_window_present (GTK_WINDOW (curr_data->about_dialog));
 	return;
   }
   
-  file = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_PIXMAP, "charpick.png", FALSE, NULL);
+  file = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_PIXMAP,
+				    "charpick.png", FALSE, NULL);
   pixbuf = gdk_pixbuf_new_from_file (file, &error);
   g_free (file);
    
   if (error) {
-  	  g_warning (G_STRLOC ": cannot open %s: %s", file, error->message);
-	  g_error_free (error);
+	g_warning (G_STRLOC ": cannot open %s: %s", file, error->message);
+	g_error_free (error);
   }
   
-  about_box = gnome_about_new (_("Character Palette"),
-			       VERSION,
-			       _("Copyright (C) 1998"),
-			       _("Gnome Panel applet for selecting strange "
-			         "characters that are not on my keyboard. "
-				 "Released under GNU General Public Licence."),
-			       authors,
-			       documenters,
-			       strcmp (translator_credits, "translator_credits") != 0 ? translator_credits : NULL,
-			       pixbuf);
+  curr_data->about_dialog = gnome_about_new (_("Character Palette"),
+					     VERSION,
+					     _("Copyright (C) 1998"),
+					     _("Gnome Panel applet for selecting strange "
+					     "characters that are not on my keyboard. "
+					     "Released under GNU General Public Licence."),
+					     authors,
+					     documenters,
+					     strcmp (translator_credits, "translator_credits") != 0 ? translator_credits : NULL,
+					     pixbuf);
   
   if (pixbuf) 
   	gdk_pixbuf_unref (pixbuf);
-   
-  gtk_window_set_screen (GTK_WINDOW (about_box),
-			 gtk_widget_get_screen (curr_data->applet));
-  gtk_window_set_wmclass (GTK_WINDOW (about_box), "character palette", "Character Palette");
-  gnome_window_icon_set_from_file (GTK_WINDOW (about_box), GNOME_ICONDIR"/charpick.png");
 
-  gtk_signal_connect(GTK_OBJECT(about_box), "destroy",
-		     GTK_SIGNAL_FUNC(gtk_widget_destroyed), &about_box);
-  gtk_widget_show(about_box);
+  gtk_window_set_screen (GTK_WINDOW (curr_data->about_dialog),
+			 gtk_widget_get_screen (curr_data->applet));
+  gtk_window_set_wmclass (GTK_WINDOW (curr_data->about_dialog),
+			  "character palette", "Character Palette");
+  gnome_window_icon_set_from_file (GTK_WINDOW (curr_data->about_dialog),
+				   GNOME_ICONDIR"/charpick.png");
+
+  gtk_signal_connect(GTK_OBJECT(curr_data->about_dialog), "destroy",
+		     GTK_SIGNAL_FUNC(gtk_widget_destroyed),
+		     &curr_data->about_dialog);
+  gtk_widget_show(curr_data->about_dialog);
   return;
 }
 
@@ -659,6 +636,8 @@ applet_destroy (GtkWidget *widget, gpointer data)
 
   g_return_if_fail (curr_data);
    
+  if (curr_data->about_dialog)
+    gtk_widget_destroy (curr_data->about_dialog);   
   if (curr_data->propwindow)
     gtk_widget_destroy (curr_data->propwindow);
 
@@ -743,14 +722,16 @@ set_atk_name_description (GtkWidget *widget, const gchar *name,
   /* return if gail is not loaded */
   if (GTK_IS_ACCESSIBLE (aobj) == FALSE)
      return;
-  atk_object_set_name (aobj, name);
-  atk_object_set_description (aobj, description);
+  if (name)
+     atk_object_set_name (aobj, name);
+  if (description)
+     atk_object_set_description (aobj, description);
 }
 
 static void
 make_applet_accessible (GtkWidget *applet)
 {
-  set_atk_name_description (applet, _("Character Palette"), NULL);
+  set_atk_name_description (applet, _("Character Palette"), _("Insert characters"));
 }
 
 static gboolean
@@ -770,7 +751,9 @@ charpicker_applet_fill (PanelApplet *applet)
   curr_data = g_new0 (charpick_data, 1);
   curr_data->last_index = NO_LAST_INDEX;
   curr_data->applet = GTK_WIDGET (applet);
-  
+  curr_data->about_dialog = NULL;
+  curr_data->add_edit_dialog = NULL;
+ 
   get_chartable (curr_data);
   
   string  = panel_applet_gconf_get_string (applet, "current_list", NULL);
@@ -824,8 +807,8 @@ charpicker_applet_fill (PanelApplet *applet)
   g_signal_connect (G_OBJECT (applet), "change_orient",
 		    G_CALLBACK (applet_change_orient), curr_data);
 
-  g_signal_connect (G_OBJECT (applet), "change_size",
-		    G_CALLBACK (applet_change_pixel_size), curr_data);
+  g_signal_connect (G_OBJECT (applet), "size_allocate",
+		    G_CALLBACK (applet_size_allocate), curr_data);
 		    
   g_signal_connect (G_OBJECT (applet), "destroy",
   		    G_CALLBACK (applet_destroy), curr_data);
@@ -850,7 +833,7 @@ charpicker_applet_fill (PanelApplet *applet)
 					NULL);
   }
 
-			             
+  register_stock_for_edit ();			             
   populate_menu (curr_data);
   
   return TRUE;
