@@ -27,83 +27,55 @@
 
 #define UPDATE_TIMEOUT 100
 
+/* FIXME crashes, need to fix */
 static void
-applet_set_back_pixmap (GtkWidget *widget, const gchar *pixmap)
+applet_set_back_pixmap (PanelApplet *applet, GtkWidget *widget, GdkPixmap *pixmap)
 {
-#ifdef FIXME    
-        GdkPixbuf *pixbuf;
-        GdkPixmap *pm;
-        GtkStyle *new_style;
-        if (!pixmap || strcmp (pixmap, "") == 0) {
-                new_style = gtk_style_copy (widget->style);
-                gtk_style_ref (new_style);
-                pm = new_style->bg_pixmap [GTK_STATE_NORMAL];
+	GtkStyle *new_style;
 
-                if (pm) 
-                        gdk_imlib_free_pixmap (pm);
-
-                new_style->bg_pixmap [GTK_STATE_NORMAL] = NULL;
-                gtk_widget_set_style (widget, new_style);
-                gtk_style_unref (new_style);
-                return;
-        }
-        if (!g_file_exists (pixmap)) 
-                return;
-        
-        pixbuf = gdk_pixbuf_new_from_file (pixmap, NULL);
-        /*imlib_image = gdk_imlib_load_image (pixmap);*/
-        if (!pixbuf)
-                return;
-        
-        /*gdk_imlib_render (imlib_image, 
-	  imlib_image->rgb_width,
-	  imlib_image->rgb_height);*/
-        /*pm = gdk_imlib_move_image (imlib_image);*/
-        gdk_pixbuf_render_pixmap_and_mask (pixbuf,&pm,NULL,0);
         new_style = gtk_style_copy (widget->style);
         gtk_style_ref (new_style);
-
-        if (new_style->bg_pixmap [GTK_STATE_NORMAL]) 
-                gdk_imlib_free_pixmap (new_style->bg_pixmap [GTK_STATE_NORMAL]);
-
-        new_style->bg_pixmap [GTK_STATE_NORMAL] = pm;
+/* FIXME should this be done ? */
+/*
+        if (new_style->bg_pixmap [GTK_STATE_NORMAL])
+		g_object_unref (new_style->bg_pixmap [GTK_STATE_NORMAL]);
+*/
+        new_style->bg_pixmap [GTK_STATE_NORMAL] = pixmap;
         gtk_widget_set_style (widget, new_style);
         gtk_style_unref (new_style);
-        gdk_pixbuf_unref (pixbuf);
-#endif
 }
 
 static void
-applet_back_change (PanelApplet *a,
-		    PanelAppletBackgroundType  type,
-		    GdkColor                  *color,
-		    const gchar               *pixmap,
-		    EyesApplet                *eyes_applet) 
+applet_back_change (PanelApplet			*a,
+		    PanelAppletBackgroundType	type,
+		    GdkColor			*color,
+		    GdkPixmap			*pixmap,
+		    EyesApplet			*eyes_applet) 
 {
 	GtkRcStyle *rc_style = gtk_rc_style_new ();
-	gint i;
-                
+
 	switch (type) {
 	case PANEL_PIXMAP_BACKGROUND:
-                applet_set_back_pixmap (eyes_applet->vbox, pixmap);
+		gtk_widget_modify_style (GTK_WIDGET (eyes_applet->applet), rc_style);
+/* FIXME can't get to work */
+//		applet_set_back_pixmap (a, GTK_WIDGET (eyes_applet->applet), pixmap);
 		break;
-        case PANEL_COLOR_BACKGROUND:
-                for (i = 0; i < eyes_applet->num_eyes; i++) {
-                	gtk_widget_modify_bg (eyes_applet->eyes[i],
-					      GTK_STATE_NORMAL,
-			    	       	      color);
-        	}
+
+	case PANEL_COLOR_BACKGROUND:
+		gtk_widget_modify_bg (GTK_WIDGET (eyes_applet->applet), GTK_STATE_NORMAL, color);
 		break;
+
+	case PANEL_NO_BACKGROUND:
+		gtk_widget_modify_style (GTK_WIDGET (eyes_applet->applet), rc_style);
+		break;
+
 	default:
-                for (i = 0; i < eyes_applet->num_eyes; i++) {
-                	gtk_widget_modify_style (eyes_applet->eyes[i], rc_style);
-        	}
+		gtk_widget_modify_style (GTK_WIDGET (eyes_applet->applet), rc_style);
 		break;
 	}
-	gtk_rc_style_unref (rc_style);
-	return;
-}
 
+	gtk_rc_style_unref (rc_style);
+}
 
 /* TODO - Optimize this a bit */
 static void 
@@ -230,6 +202,9 @@ about_cb (BonoboUIComponent *uic,
 	const gchar *translator_credits = _("translator_credits");
 
 	if (eyes_applet->about_dialog) {
+		gtk_window_set_screen (GTK_WINDOW (eyes_applet->about_dialog),
+				       gtk_widget_get_screen (GTK_WIDGET (eyes_applet->applet)));
+		
 		gtk_window_present (GTK_WINDOW (eyes_applet->about_dialog));
 		return;
 	}
@@ -294,6 +269,8 @@ setup_eyes (EyesApplet *eyes_applet)
         eyes_applet->hbox = gtk_hbox_new (FALSE, 0);
         gtk_box_pack_start (GTK_BOX (eyes_applet->vbox), eyes_applet->hbox, TRUE, TRUE, 0);
 
+	eyes_applet->eyes = g_new0 (GtkWidget *, eyes_applet->num_eyes);
+
         for (i = 0; i < eyes_applet->num_eyes; i++) {
                 eyes_applet->eyes[i] = gtk_image_new ();
                 if (eyes_applet->eyes[i] == NULL)
@@ -335,6 +312,8 @@ destroy_eyes (EyesApplet *eyes_applet)
 {
 	gtk_widget_destroy (eyes_applet->hbox);
 	eyes_applet->hbox = NULL;
+
+	g_free (eyes_applet->eyes);
 }
 
 static EyesApplet *
@@ -356,6 +335,8 @@ destroy_cb (GtkObject *object, EyesApplet *eyes_applet)
 	g_return_if_fail (eyes_applet);
 
 	gtk_timeout_remove (eyes_applet->timeout_id);
+	if (eyes_applet->hbox)
+		destroy_eyes (eyes_applet);
 	if (eyes_applet->tooltips)
 		g_object_unref (eyes_applet->tooltips);
 	eyes_applet->timeout_id = 0;
@@ -492,7 +473,6 @@ geyes_applet_fill (PanelApplet *applet)
 
 	setup_eyes (eyes_applet);
 
-
 	return TRUE;
 }
 
