@@ -47,7 +47,7 @@ typedef enum {
 	PIX_SIGNAL_2,
 	PIX_SIGNAL_3,
 	PIX_SIGNAL_4,
-	PIX_NUMBER,
+	PIX_NUMBER
 } PixmapState;
 
 static char * pixmap_names[] = {
@@ -141,7 +141,7 @@ wireless_applet_draw (WirelessApplet *applet, int percent)
 
 	/* Update the percentage */
 	if (percent > 0) {
-		tmp = g_strdup_printf ("%2.0d%%", percent);
+		tmp = g_strdup_printf ( _("Link Strength: %2.0d%%"), percent);
 	} else {
 		tmp = g_strdup_printf (_("N/A"));
 	}
@@ -250,6 +250,7 @@ wireless_applet_read_device_state (WirelessApplet *applet)
 	double link;
 	char device[256];
 	char line[256];
+	gboolean found = FALSE;
 
 	/* resest list of available wireless devices */
 	g_list_foreach (applet->devices, (GFunc)g_free, NULL);
@@ -288,6 +289,7 @@ wireless_applet_read_device_state (WirelessApplet *applet)
 				ptr++;
 
 				wireless_applet_update_state (applet, device, link, level, noise);
+				found = TRUE;
 			}
 		}
 	} while (1);
@@ -295,7 +297,7 @@ wireless_applet_read_device_state (WirelessApplet *applet)
 	if (g_list_length (applet->devices)==1) {
 		wireless_applet_set_device (applet,
 				(char*)applet->devices->data);
-	} else {
+	} else if (found == FALSE) {
 		wireless_applet_update_state (applet,
 				applet->device, -1, -1, -1);
 	}
@@ -360,10 +362,6 @@ start_file_read (WirelessApplet *applet)
 {
 	applet->file = fopen ("/proc/net/wireless", "rt");
 	if (applet->file == NULL) {
-		gtk_tooltips_set_tip (applet->tips,
-				GTK_WIDGET (applet),
-				_("No Wireless Devices"),
-				NULL);
 		show_error_dialog (_("There doesn't seem to be any wireless devices configured on your system.\nPlease verify your configuration if you think this is incorrect."));
 	}
 }
@@ -596,6 +594,9 @@ wireless_applet_about_cb (BonoboUIComponent *uic, WirelessApplet *applet)
 
 	g_object_unref (pixbuf);
 
+	gtk_window_set_screen (GTK_WINDOW (applet->about_dialog),
+			       gtk_widget_get_screen (GTK_WIDGET (applet)));
+
 	g_signal_connect (applet->about_dialog, "destroy",
 			  G_CALLBACK (gtk_widget_destroyed),
 			  &applet->about_dialog);
@@ -751,6 +752,33 @@ static void change_orient_cb(PanelApplet *pa, gint s, WirelessApplet *applet)
 	wireless_applet_timeout_handler (applet);
 }
 
+static void change_background_cb(PanelApplet *a, PanelAppletBackgroundType type,
+				 GdkColor *color, GdkPixmap *pixmap,
+				 WirelessApplet *applet)
+{
+	GtkRcStyle *rc_style = gtk_rc_style_new ();
+
+	switch (type) {
+		case PANEL_PIXMAP_BACKGROUND:
+			gtk_widget_modify_style (GTK_WIDGET (applet), rc_style);
+			break;
+
+		case PANEL_COLOR_BACKGROUND:
+			gtk_widget_modify_bg (GTK_WIDGET (applet), GTK_STATE_NORMAL, color);
+			break;
+
+		case PANEL_NO_BACKGROUND:
+			gtk_widget_modify_style (GTK_WIDGET (applet), rc_style);
+			break;
+
+		default:
+			gtk_widget_modify_style (GTK_WIDGET (applet), rc_style);
+			break;
+	}
+
+	gtk_rc_style_unref (rc_style);
+}
+
 static GtkWidget *
 wireless_applet_new (WirelessApplet *applet)
 {
@@ -794,6 +822,11 @@ wireless_applet_new (WirelessApplet *applet)
 	start_file_read (applet);
 	wireless_applet_timeout_handler (applet);
 
+	gtk_tooltips_set_tip (applet->tips,
+			      GTK_WIDGET (applet),
+			      _("No Wireless Devices"),
+			      NULL);
+
 	applet->timeout_handler_id = gtk_timeout_add
 		(CFG_UPDATE_INTERVAL * 1000,
 		 (GtkFunction)wireless_applet_timeout_handler,
@@ -807,10 +840,12 @@ wireless_applet_new (WirelessApplet *applet)
 	}
 		 
 	g_signal_connect (G_OBJECT (applet), "change_size",
-				  G_CALLBACK (change_size_cb), applet);
+			  G_CALLBACK (change_size_cb), applet);
 	g_signal_connect (G_OBJECT (applet), "change_orient",
-				  G_CALLBACK (change_orient_cb), applet);
-  
+			  G_CALLBACK (change_orient_cb), applet);
+ 	g_signal_connect (G_OBJECT (applet), "change_background",
+			  G_CALLBACK (change_background_cb), applet);
+
 	return GTK_WIDGET (applet);
 }
 
